@@ -12,7 +12,7 @@ maintaining the widget state and invoking callbacks, all aspects
 of the widgets appearance lies at Themes.
 """
 
-__version__ = "0.1.6"
+__version__ = "0.2"
 
 __author__ = "Guilherme Polo <ggpolo@gmail.com>"
 
@@ -54,7 +54,7 @@ def _loadttk(loadtk):
 Tkinter.Tk._loadtk = _loadttk(Tkinter.Tk._loadtk)
 
 def _format_optdict(optdict, script=False, ignore=None):
-    """Formats optdict to pass it to tk.call.
+    """Formats optdict to a tuple to pass it to tk.call.
     
     E.g. (script=False):
       {'foreground': 'blue', 'padding': [1, 2, 3, 4]} returns:
@@ -157,7 +157,7 @@ def _format_elemcreate(etype, script=False, *args, **kw):
 
     return spec, opts
 
-def _format_layoutlist(layout, indent=2, indent_size=2):
+def _format_layoutlist(layout, indent=0, indent_size=2):
     """Formats a layout list so we can pass the result to ttk::style
     layout and ttk::style settings. Note that the layout doesn't has to
     be a list necessarily.
@@ -196,7 +196,8 @@ def _format_layoutlist(layout, indent=2, indent_size=2):
         if "children" in opts:
             script.append(head + " -children {")
             indent += indent_size
-            newscript, indent  = _format_layoutlist(opts['children'], indent)
+            newscript, indent = _format_layoutlist(opts['children'], indent,
+                indent_size)
             script.append(newscript)
             indent -= indent_size
             script.append('%s}' % (' ' * indent))
@@ -215,11 +216,11 @@ def _script_from_settings(settings):
     for name, opts in settings.iteritems():
         # will format specific keys according to Tcl code
         if opts.get('configure'): # format 'configure'
-            s = ' '.join(map(str, _format_optdict(opts['configure'], True)))
+            s = ' '.join(map(unicode, _format_optdict(opts['configure'], True)))
             script.append("ttk::style configure %s %s;" % (name, s))
 
         if opts.get('map'): # format 'map'
-            s = ' '.join(map(str, _format_mapdict(opts['map'], True)))
+            s = ' '.join(map(unicode, _format_mapdict(opts['map'], True)))
             script.append("ttk::style map %s %s;" % (name, s))
 
         if 'layout' in opts: # format 'layout' which may be empty
@@ -1052,6 +1053,16 @@ class Scale(Widget, Tkinter.Scale):
         Widget.__init__(self, master, "ttk::scale", kw)
 
 
+    def configure(self, **kw):
+        """Modify or query scale options.
+
+        Changing "from", "from_" or "to" options generates a
+        <<RangeChanged>> event."""
+        Widget.configure(self, **kw)
+        if any(['from' in kw, 'from_' in kw, 'to' in kw]):
+            self.event_generate('<<RangeChanged>>')
+
+
     def get(self, x=None, y=None):
         """Get the current value of the value option, or the value
         corresponding to the coordinates x, y if they are specified.
@@ -1153,11 +1164,11 @@ class Treeview(Widget):
     def get_children(self, item=None):
         """Returns a tuple of children belonging to item.
         
-        If item is not specified, returns all root children."""
-        return self.tk.call(self._w, "children", item or '')
+        If item is not specified, returns root children."""
+        return self.tk.call(self._w, "children", item or '') or ()
 
 
-    def set_children(self, item, newchildren):
+    def set_children(self, item, *newchildren):
         """Replaces item's child with newchildren.
 
         Children present in item that are not present in newchildren
@@ -1182,7 +1193,7 @@ class Treeview(Widget):
         self.tk.call(self._w, "delete", items)
 
 
-    def detach(self, items):
+    def detach(self, *items):
         """Unlinks all of the specified items from the tree.
 
         The items and all of their descendants are still present, and may
@@ -1192,9 +1203,9 @@ class Treeview(Widget):
 
 
     def exists(self, item):
-        """Returns 1 if the specified item is present in the three,
-        0 otherwise."""
-        return self.tk.call(self._w, "exists", item)
+        """Returns True if the specified item is present in the three,
+        False otherwise."""
+        return bool(self.tk.call(self._w, "exists", item))
 
 
     def focus(self, item=None):
@@ -1409,7 +1420,9 @@ class Treeview(Widget):
     def tag_has(self, tagname, item=None):
         """If item is specified, returns 1 or 0 depending on whether the
         specified item has the given tagname. Otherwise, returns a list of
-        all items which have the specified tag."""
+        all items which have the specified tag.
+
+        * Availability: Tk 8.6"""
         return self.tk.call(self._w, "tag", "has", tagname, item)
 
 
@@ -1433,9 +1446,9 @@ class LabeledScale(Frame, object):
     can be accessed through instance.label"""
 
     def __init__(self, master=None, variable=None, from_=0, to=10, **kw):
-        """Construct a LabeledScale with parent master, a variable to be
-        associated with the Ttk Scale widget and its range. If variable is
-        not specified, a Tkinter.IntVar is created.
+        """Construct an horizontal LabeledScale with parent master, a
+        variable to be associated with the Ttk Scale widget and its range.
+        If variable is not specified, a Tkinter.IntVar is created.
 
         WIDGET-SPECIFIC OPTIONS
 
@@ -1445,10 +1458,13 @@ class LabeledScale(Frame, object):
         """
         self._label_top = kw.pop('compound', 'top') == 'top'
         Frame.__init__(self, master, **kw)
-        self._variable = variable or Tkinter.IntVar(master, value=from_)
+        self._variable = variable or Tkinter.IntVar(master)
+        self._variable.set(from_)
+        self._last_valid = from_
 
         self.label = Label(self)
         self.scale = Scale(self, variable=self._variable, from_=from_, to=to)
+        self.scale.bind('<<RangeChanged>>', self._adjust)
 
         # position scale and label according to the compound option
         scale_side = 'bottom' if self._label_top else 'top'
@@ -1459,27 +1475,41 @@ class LabeledScale(Frame, object):
 
         # update the label as scale or variable changes
         self._variable.trace_variable('w', self._adjust)
-        self.scale.bind('<Configure>', self._adjust)
-        self.scale.bind('<Map>', self._adjust)
+        self.bind('<Configure>', self._adjust)
+        self.bind('<Map>', self._adjust)
+
+
+    def destroy(self):
+        """Destroy this widget and possibly its associated variable."""
+        del self._variable
+        Frame.destroy(self)
 
 
     def _adjust(self, *args):
         """Adjust the label position according to the scale."""
+        def adjust_label():
+            self.update_idletasks() # "force" scale redraw
+
+            x, y = self.scale.coords()
+            if self._label_top:
+                y = self.scale.winfo_y() - self.label.winfo_reqheight()
+            else:
+                y = self.scale.winfo_reqheight() + self.label.winfo_reqheight()
+
+            self.label.place_configure(x=x, y=y)
+
+        from_, to = self.scale['from'], self.scale['to']
+        if to < from_:
+            from_, to = to, from_
         newval = self._variable.get()
-        if not self.scale['from'] <= newval <= self.scale['to']:
+        if not from_ <= newval <= to:
+            # value outside range, set value back to the last valid one
+            self.value = self._last_valid
             return
 
-        self.update()
+        self._last_valid = newval
         self.label['text'] = newval
-
-        x, y = self.scale.coords()
-        if self._label_top:
-            y = self.scale.winfo_y() - self.label.winfo_reqheight()
-        else:
-            y = self.scale.winfo_reqheight() + self.label.winfo_reqheight()
-        x = x + self.scale.winfo_x()
-
-        self.label.place_configure(x=x, y=y)
+        self.after_idle(adjust_label)
 
 
     def _get_value(self):
@@ -1496,32 +1526,41 @@ class LabeledScale(Frame, object):
 
 
 class OptionMenu(Menubutton):
-    """Themed OptionMenu which allows the user to select a value from a
-    menu."""
+    """Themed OptionMenu, based after Tkinter's OptionMenu, which allows
+    the user to select a value from a menu."""
 
     def __init__(self, master, variable, default=None, *values, **kwargs):
-        """Construct a themed OptionMenu widget with the parent master,
+        """Construct a themed OptionMenu widget with master as the parent,
         the resource textvariable set to variable, the initially selected
-        value specified by the default parameter, the other menu values
-        given by *values and an additional keyword argument command."""
+        value specified by the default parameter, the menu values given by
+        *values and additional keywords.
+
+        WIDGET-SPECIFIC OPTIONS
+
+            style: stylename
+                Menubutton style.
+            direction: 'above', 'below', 'left', 'right', or 'flush'
+                Menubutton direction.
+            command: callback
+                A callback that will be invoked after selecting an item.
+        """
         kw = {'textvariable': variable, 'style': kwargs.pop('style', None),
               'direction': kwargs.pop('direction', None)}
         Menubutton.__init__(self, master, **kw)
+        self['menu'] = Tkinter.Menu(self, tearoff=False)
 
-        self._menu = Tkinter.Menu(self, name="menu", tearoff=0)
-        self._callback = kwargs.pop('command', None)
         self._variable = variable
+        self._callback = kwargs.pop('command', None)
         if kwargs:
             raise Tkinter.TclError('unknown option -%s' % (
                 kwargs.iterkeys().next()))
 
-        self['menu'] = self._menu
-        self.set_menu(default, *((default, ) + values))
+        self.set_menu(default, *values)
 
 
     def __getitem__(self, item):
         if item == 'menu':
-            return self._menu
+            return self.nametowidget(Menubutton.__getitem__(self, item))
 
         return Menubutton.__getitem__(self, item)
 
@@ -1540,6 +1579,6 @@ class OptionMenu(Menubutton):
 
 
     def destroy(self):
-        """Destroy this widget and the associated menu."""
+        """Destroy this widget and its associated variable."""
+        del self._variable
         Menubutton.destroy(self)
-        self._menu = None
